library ieee;
use ieee.std_logic_1164.all;

entity ResetSyncer is
	generic (
		g_Length: integer := 3;
		g_Inverted: boolean := FALSE
	);
	port (
		Clk_ik: in std_logic;
		Reset_ira: in std_logic;
		Reset_or: out std_logic
	);
end entity;

architecture syn of ResetSyncer is

	function if_sl(condition: boolean; pos: std_logic; neg: std_logic) return std_logic is begin
		if condition then
			return pos;
		else
			return neg;
		end if;
	end function;

	attribute ASYNC_REG: string;
	
	constant c_Inversion: std_logic := if_sl(g_Inverted, '1', '0');

	signal ShiftRegister: std_logic_vector(g_Length-1 downto 0) := (others => ('1' xor c_Inversion));

	attribute ASYNC_REG of ShiftRegister: signal is "TRUE";

begin

	pSyncer: process (Clk_ik, Reset_ira) begin
		if Reset_ira = ('1' xor c_Inversion) then
			ShiftRegister <= (others => ('1' xor c_Inversion));
		elsif rising_edge(Clk_ik) then
			ShiftRegister <= ShiftRegister(g_Length-2 downto 0)&('0' xor c_Inversion);
		end if;
	end process;
	
	Reset_or <= ShiftRegister(g_Length-1);

end architecture;
