`include "timescale.svh"
`include "Config.svh"

import uvm_pkg::*;
import Ad5600Package::*;
import TestbenchPackage::*;

module Testbench;

    parameter c_ClkPeriod = 1e9/`CLK_FREQ; // in [ns]
    
    bit Clk_k, Reset_r;

    FfpgInterface LocalInterface(Clk_k, Reset_r);

    FfpgCoreWrapper #(
        .g_ClkFrequency(`CLK_FREQ)
    ) dut (
        .rst_i(Reset_r),
        .clk_sys_i(Clk_k),
        .wb_adr_i(LocalInterface.Wb.adr),
        .wb_dat_i(LocalInterface.Wb.dat_o),
        .wb_dat_o(LocalInterface.Wb.dat_i),
        .wb_cyc_i(LocalInterface.Wb.cyc),
        .wb_sel_i(LocalInterface.Wb.sel),
        .wb_stb_i(LocalInterface.Wb.stb),
        .wb_we_i(LocalInterface.Wb.we),
        .wb_ack_o(LocalInterface.Wb.ack),
        .wb_err_o(LocalInterface.Wb.err),
        .wb_rty_o(LocalInterface.Wb.rty),
        .wb_stall_o(), // doesn't exist in B3 version of WB
        .wb_int_o(), // doesn't exist in B3 version of WB
        .ClkIn0_ik(LocalInterface.Fmc.ClkIn0),
        .TriggerDacFrameSynchronization_n(LocalInterface.TriggerDac.FrameSynchronization_n),
        .TriggerDacSerialClock(LocalInterface.TriggerDac.SerialClock),
        .TriggerDacSerialData(LocalInterface.TriggerDac.SerialData),
        .VcxoDacFrameSynchronization_n(LocalInterface.VcxoDac.FrameSynchronization_n),
        .VcxoDacSerialClock(LocalInterface.VcxoDac.SerialClock),
        .VcxoDacSerialData(LocalInterface.VcxoDac.SerialData),
         // not tested so far:
        .Ch1OutputEnable_o(),
        .Ch2OutputEnable_o(),
        .DelayValue_ob(),
        .Ch1SetLe_o(),
        .Ch1ResLe_o(),
        .Ch2SetLe_o(),
        .Ch2ResLe_o(),
        .Ch1Set_o(),
        .Ch1Res_o(),
        .Ch2Set_o(),
        .Ch2Res_o(),
        .Trigger_i(LocalInterface.Fmc.Trigger),
        .SpiAd9512Sclk_o(),
        .SpiAd9512Mosi_o(),
        .SpiAd9512Miso_i(),
        .SpiAd9512Cs_on(),
        .Clk2Sel_o(),
        .Onewire_io(),
        .Led_ob()
    );
    
    initial begin: clockGenerator
        Clk_k = 0;
        forever #(c_ClkPeriod/2) Clk_k = ~Clk_k;
    end

    initial begin: resetGenerator
        Reset_r = 1;
        repeat(3) @(negedge Clk_k);
        Reset_r = 0;
    end
    assign LocalInterface.Wb.rst_i = Reset_r;

    initial begin: mainTest
        uvm_config_db #(virtual FfpgInterface)::set(null, "uvm_test_top.*", "dut_vi", LocalInterface);
        uvm_top.finish_on_completion = 0; // not to finish when GUI is used
        // run_test("TestTriggerAdc");
        // run_test("TestDelayConfiguration");
        // run_test("TestPulseGeneration");
        run_test("TestPulseGenerationSequential");
        $stop();
    end
    
    initial begin: timeout
        #1000us $error("Simulation timeout!");
        $stop();
    end

endmodule